@@ -1,16 +1,33 @@
 # -*- coding: utf8 -*-
 import numpy
-from .. import utils, deflation
-
+from .. import utils, deflation, linsys
+from . import factories, evaluators
 
 
 class _RecyclingSolver(object):
     '''Base class for recycling solvers.'''
-    def __init__(self, DeflatedSolver):
+    def __init__(self, DeflatedSolver,
+                 vector_factory=None
+                 ):
         '''Initialize recycling solver base.
 
         :param DeflatedSolver: a deflated solver from
           :py:mod:`~krypy.deflation`.
+        :param vector_factory: (optional) An instance of a subclass of
+          :py:class:`krypy.recycling.factories._DeflationVectorFactory`
+          that constructs deflation vectors for recycling. Defaults to `None`
+          which means that no recycling is used.
+
+          Also the following strings are allowed as shortcuts:
+
+           * ``'RitzApproxKrylov'``: uses the approximate Krylov subspace bound
+             evaluator :py:class:`krypy.recycling.evaluators.RitzApproxKrylov`.
+           * ``'RitzAprioriCg'``: uses the CG :math:`\kappa`-bound
+             (:py:class:`krypy.utils.BoundCG`) as an a priori bound with
+             :py:class:`krypy.recycling.evaluators.RitzApriori`.
+           * ``'RitzAprioriMinres'``: uses the MINRES bound
+             (:py:class:`krypy.utils.BoundMinres`) as an a priori bound with
+             :py:class:`krypy.recycling.evaluators.RitzApriori`.
 
         After a run of the provided ``DeflatedSolver`` via :py:meth:`solve`,
         the resulting instance of the ``DeflatedSolver`` is available in the
@@ -18,6 +35,8 @@ class _RecyclingSolver(object):
         '''
         self._DeflatedSolver = DeflatedSolver
 
+        self._vector_factory = vector_factory
+
         self.timings = utils.Timings()
         '''Timings from last run of :py:meth:`solve`.
 
@@ -35,14 +54,12 @@ class _RecyclingSolver(object):
               *args, **kwargs):
         '''Solve the given linear system with recycling.
 
-        The deflation vectors and the Krylov subspace of the last solve() call
-        are examined and considered for deflation according to the provided
-        ``deflation_vector_factory``.
+        The provided `vector_factory` determines which vectors are used for
+        deflation.
 
         :param linear_system: the :py:class:`~krypy.linsys.LinearSystem` that
           is about to be solved.
-        :param vector_factory: (optional) An instance of a subclass
-          of :py:class:`krypy.recycling.factories._DeflationVectorFactory`.
+        :param vector_factory: (optional) see description in constructor.
 
         All remaining arguments are passed to the ``DeflatedSolver``.
 
@@ -50,7 +67,33 @@ class _RecyclingSolver(object):
           approximate solution. The approximate solution is available under the
           attribute ``xk``.
         '''
+
+        # replace linear_system with equivalent TimedLinearSystem on demand
+        if not isinstance(linear_system, linsys.TimedLinearSystem):
+            linear_system = linsys.ConvertedTimedLinearSystem(linear_system)
+
         with self.timings['vector_factory']:
+            if vector_factory is None:
+                vector_factory = self._vector_factory
+
+            # construct vector_factory if strings are provided
+            if vector_factory == 'RitzApproxKrylov':
+                vector_factory = factories.RitzFactory(
+                    subset_evaluator=evaluators.RitzApproxKrylov()
+                    )
+            elif vector_factory == 'RitzAprioriCg':
+                vector_factory = factories.RitzFactory(
+                    subset_evaluator=evaluators.RitzApriori(
+                        Bound=utils.BoundCG
+                        )
+                    )
+            elif vector_factory == 'RitzAprioriMinres':
+                vector_factory = factories.RitzFactory(
+                    subset_evaluator=evaluators.RitzApriori(
+                        Bound=utils.BoundMinres
+                        )
+                    )
+
             # get deflation vectors
             if self.last_solver is None or vector_factory is None:
                 U = numpy.zeros((linear_system.N, 0))
